# Generated by Django 4.2.27 on 2026-01-07 09:19

# Django
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("esistatus", "0002_openapi_status"),
    ]

    operations = [
        migrations.AddField(
            model_name="esistatus",
            name="total_endpoints",
            field=models.PositiveIntegerField(
                default=0, help_text="Total number of ESI endpoints."
            ),
        ),
    ]
