# Generated by Django 4.2.26 on 2025-11-20 20:11

# Django
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("esistatus", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="Status",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "compatibility_date",
                    models.CharField(
                        help_text="The ESI compatibility date.", max_length=10
                    ),
                ),
                (
                    "status_data",
                    models.JSONField(
                        blank=True, help_text="The ESI status data.", null=True
                    ),
                ),
            ],
            options={
                "verbose_name": "ESI Endpoint Status",
                "verbose_name_plural": "ESI Endpoint Statuses",
                "default_permissions": (),
            },
        ),
        migrations.RenameModel(old_name="Status", new_name="EsiStatus"),
    ]
