{% extends "esistatus/base.html" %}

{% load static %}
{% load i18n %}

{% block aa_esistatus_body %}
    <div class="esistatus-loading text-center">
        <svg class="svg-sprite svg-loading-spinner">
            <use href="#aa-loading-spinner"></use>
        </svg>
        <p class="my-1">{% translate "Loading ESI status…" %}</p>
    </div>

    <div class="esi-status-index"></div>
{% endblock %}

{% block extra_css %}
    {% include "esistatus/bundles/esistatus-css.html" %}
{% endblock %}

{% block extra_javascript %}
    <script>
        const esistatusSettings = {
            url: {
                esistatus: '{% url "esistatus:ajax_esi_status" %}',
            }
        };
    </script>

    {% include "esistatus/bundles/esistatus-js.html" %}
{% endblock %}
